
from sqlobject import *

import time
import mx.DateTime


class BossConfig(dict):
    def __init__(self):
        import os
        import re

        dict.__init__(self)
        #self._dict = {}

        configFileName = "BossConfig.clad"
        bossTop = None
        homeDir = None
        possibleConfigFileNames = [configFileName]
        try:
            homeDir = os.environ['HOME']
            possibleConfigFileNames.append(os.path.join(homeDir,configFileName))
        except: pass
        try:
            bossTop = os.environ['BOSSDIR']
            possibleConfigFileNames.append(os.path.join(bossTop,configFileName))
        except: pass

        f = None
        for configFile in possibleConfigFileNames:
            try:
                f = open(configFile,'r')
            except: pass

        if not f:
            raise Exception("Cannot find BOSS configuration")

        lineNo = 0
        configPat = re.compile(r'^\s*(\w+)\s*=\s*("?[\w./*|;:!@$%=+-_]+"?)\s*;')
        for line in f.readlines():
            lineNo = lineNo + 1
            # Nuke comments
            commentStart = line.find('#')
            if commentStart >= 0:
                realLine = line[:commentStart]
                pass
            else:
                realLine = line
            m = configPat.match(realLine)
            if m:
                try:
                    self[m.group(1)] = eval(m.group(2))
                except:
                    print "Bad BOSSConfig line %d"%lineNo, line

        f.close()

        # Try to set up the database connection string thingee.
        try:
            host =  self['BOSS_DB_HOST']
            user = self['BOSS_DB_USER']
            passwd =  self['BOSS_DB_USER_PW']
            database = self['BOSS_DB_NAME']
            self['conn'] = "mysql://%s:%s@%s/%s" % (user, passwd, host, database)
        except:
            self['conn'] = None


try:
    config = BossConfig()
except:
    print "WARNING: No BOSS Config"
    config = None


def _timeToTicks(value):
    try:
        v = int(value)
    except:
        v = mx.DateTime.DateTimeFrom(value)
        v = int(v.ticks())
    return v


class BossJob (SQLObject):
    _table = "JOB"
    try:    _connection = config['conn']
    except: _connection = None
    type         = StringCol(length=30)
    sch          = StringCol(length=30)
    sid          = StringCol(length=100)
    s_host       = StringCol(length=30)
    s_path       = StringCol(length=255)
    s_usr        = StringCol(length=30)
    executable   = StringCol(length=255, dbName='exec')
    args         = StringCol(length=255)
    stdin        = StringCol(length=255)
    stdout       = StringCol(length=255)
    stderr       = StringCol(length=255)
    e_host       = StringCol(length=30)
    e_path       = StringCol(length=255)
    e_usr        = StringCol(length=30)
    log          = StringCol(length=255)
    t_sub        = IntCol()
    t_start      = IntCol()
    t_stop       = IntCol()
    t_stat       = StringCol(length=50)
    ret_code     = StringCol(length=11)

    # BOSS is using raw unix seconds since epoch time idiocy
    # instead of native DB time types.  KILL ME NOW.
    #
    # Unhappily, Condor uses at least two different string
    # representations for time, so we can't do a cheap conversion
    # with python's native time module.  Probably a good thing as it
    # forces us to at least try to do time correctly.  A bad
    # thing because -- ARGH!!!

    # If we have mx (which we don't) we could use this nugget:
    #
    # >>> import mx.DateTime
    # >>> t= mx.DateTime.DateTimeFrom("11/09 14:28:35")
    # >>> print t
    # 2004-11-09 14:28:35.00
    # >>> t.ticks()
    # 1100032115.0


    def _set_t_sub(self, value):
        self._SO_set_t_sub(_timeToTicks(value))

    def _set_t_start(self, value):
        self._SO_set_t_start(_timeToTicks(value))

    def _set_t_stop(self, value):
        self._SO_set_t_stop(_timeToTicks(value))

