############################################################################
# Noah J Edelson, LBNL
# See Copyright for copyright notice!
###########################################################################

import sys, exceptions

from vipus.dag.job import Job

class MissingOutputException(Exception): pass

def write_job(writefilename, config):
    writefile = open(writefilename, "w")
    for key in 'executable', 'error', 'log', 'universe', 'notification', 'getenv', 'log_xml':
        writefile.write("%s=%s\n" % (key, config[key]))
    if len(config['arguments']) > 0:
        writefile.write("arguments=")
        for i in range(len(config['arguments'])):
            writefile.write(config['arguments'][i])
            if i < len(config['arguments']) - 1:
                writefile.write(" ")
        writefile.write("\n")
    if config.has_key('input'):
        writefile.write("input=%s\n" % str(str(str(config['input']).replace('\'', ' ')).replace('[',' ').replace(']', ' ')).replace(' ',''))
    if config.has_key('output'):
        writefile.write("output=%s\n" % (config['output']))
    if config.has_key('initialdir'):
        writefile.write("initialdir=%s\n" % (config['initialdir']))
    writefile.write("queue\n")                          
    writefile.close()

def read_job(readfilename, config):
    readfilename = readfilename.split('\n')[0]
    readfiledata = open(readfilename,'r').readlines()
    for lines in readfiledata:
        formatted = lines.split('=')
        if len(formatted) == 2:
            config[ formatted[0] ] = formatted[1]

class CondorJob(Job):
    def __init__(self, executable, output=None, arguments=[], initialdir=None, error='default.err', log='default.log', universe='VANILLA', notification='NEVER'):
        """
        Job constructor.
        @param executable:  string containing the filename of the executable
        @param output:      string containing the filename to map stdout to.
        @param arguments:   
        @param initialdir:  string containing the running directory
        @param error:       string containing the filename to map stderr to.   (DEFAULT: default.err)
        @param log:         string containing the logfilename                  (DEFAULT: default.log)
        @param universe:    string containing the condor universe type         (DEFAULT: VANILLA)
        @param notification:string containing the notification type            (DEFAULT: NEVER)
        """
        Job.__init__(self, initialdir=initialdir, error=error, log=log)
        self.config['executable'] = executable
        if output:
            self.config['output'] = output
        if log:
            self.config['log'] = log
        self.config['arguments'] = arguments
        self.config['universe'] = universe
        self.config['notification'] = notification
        self.config['getenv'] = 'True'
        self.config['log_xml'] = 'True'

        self.pre   = ''
        self.post  = ''
        self.retry = ''

    def writeFile(self, writefilename):
        """
        Writes a .JOB file.
        @param writefilename: The .job filename to populate with the self's attributes.        
        """
        write_job(writefilename, self.config)

    def readFile(self, readfilename):
        """
        Reads a .JOB file.        
        @param readfilename: The .job filename to populate self's attributes with.
        """
        print 'readFile:  ', readfilename.split('\n')[0]
        read_job(readfilename, self.config)
            
    def attachScriptPRE(self, script):
        """
        Attach a shell script for execution prior to the given job.
        @param script: the shell script to execute
        """
        self.pre = script
        
    def attachScriptPOST(self, script):
        """
        Attach a shell script for execution posterior to the given job. (that means after)
        @param script: the shell script to execute        
        """
        self.post = script
        
    def attachRetry(self, retry):
        """
        Set number of retries (rescues?) that this job will attempt before failing.
        @param retry: number of retries that this job will attempt before failing.
        """
        self.retry = retry

    def report(self):        
        print 'report: ', self.jobfilename
        for key in 'executable', 'error', 'log', 'universe', 'notification', 'getenv', 'log_xml':
            print ("%s=%s\n" % (key, self.config[key]))
        if len(self.config['arguments']) > 0:
            print("arguments=")
            for i in range(len(self.config['arguments'])):
                print(self.config['arguments'][i])
                if i < len(self.config['arguments']) - 1:
                    print(" ")
            print("\n")
        if self.config.has_key('input'):
            print("input=%s\n" % (self.config['input']))
        if self.config.has_key('output'):
            print("output=%s\n" % (self.config['output']))
        if self.config.has_key('initialdir'):
            print("initialdir=%s\n" % (self.config['initialdir']))
        print("queue\n")                          



            
def main(argv):
    print 'vipus condor_job library, not for main()ing!'

if __name__ == "__main__" : main(sys.argv)
