# $Id: onasysd.py,v 1.17.2.1 2005/03/24 16:32:29 kipp Exp $

#
# Preamble
#

import ConfigParser
from glue import LSCsegFindClient
from glue.gpstime import GpsSecondsFromPyUTC
from glue.segments import *
import logging
import os
import pickle
import re
from stat import S_ISDIR
from tempfile import mkstemp
import time


#
# Config file interface
#

class config_file:
	def __init__(self, file = "onasysd.ini"):
		self.file = file
		self.defaults = {
			"dag_datadir" : os.getcwd(),
			"dryrun" : "True",
			"e-mail" : "",
			"initial_epoch" : 100000000,
			"job_throttle" : -1,
			"min_segment_length" : 0,
			"overlap" : 0,
			"pid_dir" : "/tmp",
			"state_file" : "onasysd.state",
			"update_interval" : 3600,
			"virtual_time" : 0
		}
		try:
			self.defaults["segfind_server"] = os.environ["LSC_SEGFIND_SERVER"]
		except KeyError:
			pass

	def parse(self):
		self.parser = ConfigParser.ConfigParser(self.defaults)
		self.parser.read(self.file)

		# [onasysd] section
		self.dryrun = self.parser.get("onasysd", "dryrun", True).lower() == "true"
		self.e_mail = str(self.parser.get("onasysd", "e-mail", True))
		self.initial_epoch = int(self.parser.get("onasysd", "initial_epoch", True))
		self.job_throttle = int(self.parser.get("onasysd", "job_throttle", True))
		self.pid_dir = str(self.parser.get("onasysd", "pid_dir", True))
		self.segfind_ifo = str(self.parser.get("onasysd", "segfind_instrument", True))
		self.segfind_server = str(self.parser.get("onasysd", "segfind_server", True))
		self.segfind_type = str(self.parser.get("onasysd", "segfind_type", True))
		self.state_file = str(self.parser.get("onasysd", "state_file", True))
		self.update_interval = int(self.parser.get("onasysd", "update_interval", True))
		self.virtual_time = int(self.parser.get("onasysd", "virtual_time", True))

		# [pipeline] section
		self.dag_base_name = str(self.parser.get("pipeline", "dag_base_name", True))
		self.dag_datadir = str(self.parser.get("pipeline", "dag_datadir", True))
		self.dag_generator = str(self.parser.get("pipeline", "dag_generator", True))
		self.min_segment_length = int(self.parser.get("pipeline", "min_segment_length", True))
		self.overlap_hack = int(self.parser.get("pipeline", "overlap", True))

	def validate(self, log):
		good = True
		if 2 * self.overlap_hack > self.min_segment_length:
			log.error("overlap exceeds 1/2 minimum segment length")
			good = False
		if len(self.dag_base_name) > 30:
			log.error("dag_base_name length exceeds 30 characters")
			good = False
		if self.pid_dir:
			if not S_ISDIR(os.stat(self.pid_dir).st_mode):
				log.error("pid_dir is not a directory")
				good = False
			elif not os.access(self.pid_dir, os.W_OK):
				log.error("pid_dir cannot be written to")
				good = False
		try:
			self.dag_base_name.index(".")
			log.error("dag_base_name contains a . character")
			good = False
		except:
			pass
		return good

	def log(self, log):
		log.info("=== BEGIN CONFIG ===")
		log.info("dryrun = " + str(self.dryrun))
		log.info("e-mail = " + str(self.e_mail))
		log.info("initial_epoch = " + str(self.initial_epoch))
		log.info("job_throttle = " + str(self.job_throttle))
		log.info("pid_dir = " + str(self.pid_dir))
		log.info("segfind_server = " + str(self.segfind_server))
		log.info("segfind_type = " + str(self.segfind_type))
		log.info("state_file = " + str(self.state_file))
		log.info("update_interval = " + str(self.update_interval))
		log.info("virtual_time = " + str(self.virtual_time))
		log.info("dag_base_name = " + str(self.dag_base_name))
		log.info("dag_datadir = " + str(self.dag_datadir))
		log.info("dag_generator = " + str(self.dag_generator))
		log.info("min_segment_length = " + str(self.min_segment_length))
		log.info("overlap = " + str(self.overlap_hack))
		log.info("=== END CONFIG ===")


#
# Crazy e-mail sending hack
#

class administrator:
	def __init__(self, address):
		self.set_address(address)

	def set_address(self, address):
		self.address = str(address)

	def set_subject(self, subject):
		self.subject = str(subject)

	def msg(self, msg):
		if self.address:
			o, i = os.popen2("mail -s " + self.subject + " " + self.address)
			o.write(msg)
			o.close()


#
# Log interface
#

class log_file:
	def __init__(self, admin):
		self.log = logging.getLogger("onasysd")
		self.handler = logging.StreamHandler()
		self.handler.setFormatter(logging.Formatter("%(asctime)s: %(name)s (%(process)d): %(levelname)s: %(message)s"))
		self.log.addHandler(self.handler)
		self.log.setLevel(logging.INFO)
		self.set_admin(admin)

	def set_admin(self, admin):
		self.admin = admin

	def info(self, s):
		self.log.info(s)

	def warning(self, s):
		self.log.warning(s)

	def error(self, s):
		self.log.error(s)
		self.admin.msg(s)


#
# Saved state interface
#

class state_file:
	def __init__(self, filename):
		self.set_filename(filename)

	def set_filename(self, filename):
		self.filename = str(filename)

	def save(self, obj):
		try:
			f = file(self.filename, "w")
			pickle.dump(obj, f)
			f.close()
		except:
			raise IOError

	def restore(self):
		try:
			f = file(self.filename, "r")
			obj = pickle.load(f)
			f.close()
		except:
			raise IOError
		return obj


#
# Time calculations.
#

class clock:
	def __init__(self):
		self.start = 0
		self.virtual_start = 0

	def set_start(self, start):
		self.start = start

	def set_virtual_start(self, virtual_start):
		self.virtual_start = virtual_start

	def now(self):
		return GpsSecondsFromPyUTC(time.time())

	def maxend(self):
		if self.virtual_start:
			return self.now() - self.start + self.virtual_start
		return 999999999


#
# PID file interface
#

class pid_file:
	def __init__(self, base, pid, info, dir = ""):
		self.base = str(base)
		self.pid = pid
		self.pid_filename = ""
		self.info = state_file(None)
		try:
			if str(dir):
				(f, self.pid_filename) = mkstemp(".pid", "onasysd." + self.base + ".", str(dir))
			else:
				(f, self.pid_filename) = mkstemp(".pid", "onasysd." + self.base + ".")
			os.write(f, str(self.pid) + "\n")
			os.close(f)
		except:
			pass
		self.set_info(info)

	def set_info(self, info):
		self.info = state_file(self.pid_filename[:-3] + "info")
		try:
			self.info.save(info)
		except:
			pass

	def touch(self):
		for name in [self.pid_filename, self.info.filename]:
			try:
				os.utime(name, None)
			except:
				pass

	def remove(self):
		for name in [self.pid_filename, self.info.filename]:
			try:
				os.remove(name)
			except:
				pass


#
# Job scheduler interface
#

class scheduler:
	def __init__(self):
		self.load = -1

	def get_load(self):
		try:
			for line in os.popen("boss_q -user " + os.environ["LOGNAME"]):
				pass
			self.load = int(line.split()[-1])
		except:
			self.load = -1
		return self.load

	def submit_command(self, dag, type = ""):
		command = "boss_submit_dag " + str(dag)
		if str(type):
			command += " -jobtype " + str(type)
		return command

	def submit(self, dag, type = ""):
		return os.popen(self.submit_command(dag, type)).close() == None


#
# User's dag generator interface
#

class dag_generator:
	def __init__(self, base_name = "", data_dir = "", generator = ""):
		self.set_base_name(base_name)
		self.set_data_dir(data_dir)
		self.set_generator(generator)
		self.set_data_seg(None)

	def set_base_name(self, base_name):
		self.base_name = str(base_name)

	def set_data_dir(self, data_dir):
		self.data_dir = str(data_dir)

	def set_generator(self, generator):
		self.generator = str(generator)

	def set_data_seg(self, segment):
		self.dataseg = segment
		self.set_trig_seg(segment)

	def set_trig_seg(self, segment):
		self.trigseg = segment

	def dag_name(self):
		return self.base_name + "." + str(self.dataseg) + ".dag"

	def working_dir(self):
		return "onasysd.jobs/" + str(self.dataseg)[0:4] + "/" + str(self.dataseg)

	def generate_command(self):
		return self.data_dir + "/" + self.generator + " -s " + str(self.dataseg.start) + " -e " + str(self.dataseg.end) + " -a " + str(self.trigseg.start) + " -b " + str(self.trigseg.end) + " -f " + self.dag_name() + " -t " + self.data_dir

	def generate(self, overlap_hack):
		stdout = os.popen(self.generate_command())
		limits = map(int, stdout.readline().split())
		success = stdout.close() == None
		if len(limits) == 2:
			return segment(limits[0], limits[1]), success
		return segment(self.dataseg.start + overlap_hack, self.dataseg.end - overlap_hack), success


#
# LSCsegFind interface
#

class segfind:
	def __init__(self, instrument, type, server = ""):
		self.set_instrument(instrument)
		self.set_type(type)
		self.set_server(server)

	def set_instrument(self, instrument):
		self.instrument = str(instrument)

	def set_type(self, type):
		self.type = str(type)

	def set_server(self, server):
		self.host = str(server)
		try:
			[self.host, self.port] = self.host.split(":")
			self.port = int(self.port)
		except:
			self.port = None

	def command(self, bounds):
		command = "LSCsegFind --type " + self.type + " --interferometer " + self.instrument + " --gps-start-time " + str(bounds.start) + " --gps-end-time " + str(bounds.end)
		if self.host:
			command += " --server " + self.host
		if self.port:
			command += ":" + str(self.port)
		return command

	def list(self, bounds):
		if self.port:
			client = LSCsegFindClient.LSCsegFind(self.host, self.port)
		else:
			client = LSCsegFindClient.LSCsegFind(self.host)
		return client.findStateSegments({"interferometer" : self.instrument, "type" : self.type, "start" : str(bounds.start), "end" : str(bounds.end), "lfns" : False, "strict" : True})
